#include <dbupdate.h>
#include <pkutil.h>
#include <assassin.h>
#include <gears.h>
#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/stat.h>
#include <time.h>
#include <debug.h>


extern Ecore_Evas *ee;

static void pk_refresh_finished_cb (void *data, DBusMessage *message);
/**
 * @brief Check if the package database need to be updated or not. Here using time stamp to do this job.
 * @return 0 means no update needed, else is the package database need to be renew
 */ 
void db_update_check() {
	if (!network_check()) {
		debug("Network check error. Need to connect first.\n");
		return;
	}
	if (db_update_timestamp_check()) {
		package_database_update();
	}
	return;
}

int db_update_timestamp_check() {
	char file[MAX_PACKAGE_NAME_SIZE];
	char buf[MAX_PACKAGE_NAME_SIZE];
	struct stat s;
	int time_now,time_last;
	FILE *fp;

	snprintf(file, sizeof(file), "%s/%s", THEME_PATH, STAMPFILE);
	time_now = time(NULL);
	if (stat(file, &s) < 0) {
		debug("State file failed!!\n");
		fp = fopen(file, "w");
		if (!fp) {
			error("Open timestamp file %s failed!!\n",file);
			return 0;
		}
		fclose(fp);
		debug("Try to update for the first stamping");
		put_time_stamp(); // put dummy stamp here for debuging usage (remove me latter)
		return 1;  // Do refresh for the first time
	}
	fp = fopen(file,"r");
	if (!fp) {
		error("Open timestamp file %s failed!!\n",file);
		return 0;
	}
	fread(buf,sizeof(char),sizeof(buf),fp);
	fclose(fp);
	time_last= atoi(buf);
	debug("The timestamp is %d, now =%d\n",time_last,time_now);
	if ((time_now - time_last) > TIMEOUT_S) {
		return 1;
	}
	return 0;
}

void put_time_stamp() {
	int time_now;
	FILE *fp;
	char buf[MAX_PACKAGE_NAME_SIZE];
	char file[MAX_PACKAGE_NAME_SIZE];
	time_now = time(NULL);
	snprintf(buf,sizeof(buf),"%d",time_now);
	snprintf(file, sizeof(file), "%s/%s", THEME_PATH, STAMPFILE);
	fp = fopen(file, "w");
	if (!fp) {
		error("Open timestamp file %s failed!!\n",file);
	}
	fwrite(buf, sizeof(char),strlen(buf),fp);
	fclose(fp);
}



int network_check() {
	// TODO: Current it's dummy here 
	// Wait for NM 0.7
	return 1;
}

static int __is_datebase_updating__ = 0;
struct Installer_Progressbar * __db_refresh_progress_bar__=NULL;
void package_database_update() {
	Pk_Operation *pko;
	__is_datebase_updating__=1;
	pko = (Pk_Operation *) malloc (sizeof(Pk_Operation));
	memset(pko,0,sizeof(Pk_Operation));
	pko->Operations[Finished_sig] =         & pk_refresh_finished_cb;
	pk_refresh_system(pko);	
	put_a_mask_on_screen();
	__db_refresh_progress_bar__ = new_Progressbar(ecore_evas_get(ee),"");
	progressbar_text_set(__db_refresh_progress_bar__,"Refresh Cache...");
	progressbar_animator_start(__db_refresh_progress_bar__);
}


static void pk_refresh_finished_cb (void *data, DBusMessage *message) {
	char *tid;
	char *exit_text;
	unsigned int runtime;
	Pk_Operation * pko = (Pk_Operation *)data;
	DBusError _err;
	dbus_error_init(&_err);
	if (!dbus_message_get_args(message, &_err,
	                           DBUS_TYPE_STRING, &tid,
	                           DBUS_TYPE_STRING, &exit_text,
	                           DBUS_TYPE_UINT32,   &runtime,
	                           DBUS_TYPE_INVALID )) {
		error("Error Message: '%s'\n", _err.message);
		dbus_error_free(&_err);
		return;
	}
	dbus_error_free(&_err);
	if (!pk_should_proxy(pko, tid)) {
		return;
	}

	remove_the_mask_on_screen();

	if (__db_refresh_progress_bar__) {
		progressbar_animator_stop(__db_refresh_progress_bar__);
		free_Progressbar(__db_refresh_progress_bar__);
		__db_refresh_progress_bar__=NULL;
	}
	pk_default_get_finished_cb (data,message);
	put_time_stamp();
	__is_datebase_updating__=0;
}

int package_database_updating() {
	return __is_datebase_updating__;
}
